<?php


namespace MirfalahTech\Laravel\Payment\Gateway\SenangPay;


use Illuminate\Http\Request;
use Illuminate\Support\Facades\Response as Res;
use MirfalahTech\Laravel\Payment\Contracts\PayableEntity;
use MirfalahTech\Laravel\Payment\Contracts\PaymentGatewayDriver;
use MirfalahTech\Laravel\Payment\Facade\Payment;
use MirfalahTech\Laravel\Payment\Traits\BillStatusBoolean;
use Symfony\Component\HttpFoundation\Response;

class SenangPayDriver implements PaymentGatewayDriver
{
    use BillStatusBoolean;

    /**
     * @var string
     */
    protected $endpoint;

    /**
     * @var string
     */
    protected $merchant_id;

    /**
     * @var string
     */
    protected $secret_key;

    /**
     * @var array
     */
    protected $keyMap;

    public function __construct(string $endpoint, string $merchant_id, string $secret_key, array $keyMap)
    {
        $this->endpoint = rtrim($endpoint, '/');
        $this->merchant_id = $merchant_id;
        $this->secret_key = $secret_key;
        $this->keyMap = $keyMap;
    }

    public function createPaymentURL(PayableEntity $payable, array $options = []): string
    {
        $detail = $payable->getBillDescription();
        $amount = $payable->getBillAmount();
        $order_id = $payable->getBillId();

        $detail = preg_replace('/[^A-Z0-9,\-_]/i', '_', $detail);
        $amount = sprintf('%0.2f', $amount);
        $hash = md5($this->secret_key . $detail . $amount . $order_id);

        return "$this->endpoint/payment/$this->merchant_id?" .
            http_build_query(compact('detail', 'order_id', 'amount', 'hash'));
    }

    public function verifyGatewayReturn(Request $request): bool
    {
        $hash = $request->query->get($this->keyMap['hash']);
        return $hash == $this->computeHash($request->request->all());
    }

    public function verifyGatewayCallback(Request $request, Response &$response = null): bool
    {
        $response = Res::make('OK');
        $hash = $request->request->get($this->keyMap['hash']);
        return $hash == $this->computeHash($request->request->all());
    }

    public function getBillIdFromRequest(Request $request): ?string
    {
        return $request->get($this->keyMap['order_id']);
    }

    public function getBillStatus(Request $request): ?int
    {
        $status = $request->get($this->keyMap['status_id']);
        if ($status == null) {
            return null;
        } else {
            return $status == 1 ? Payment::SUCCESS : Payment::PENDING;
        }
    }

    public function computeHash($payload)
    {
        $query = http_build_query($payload);
        $key = preg_quote(http_build_query([$this->keyMap['hash']=>'']), '/');
        $query = preg_replace('/(^|&)'.$key.'[^&]*($|&)/','$1hash=[HASH]$2', $query);
        return md5($this->secret_key . '?' . $query);
    }
}