<?php


namespace MirfalahTech\Laravel\Payment\Gateway\SenangPay;


use Illuminate\Contracts\Config\Repository;
use Illuminate\Contracts\Container\BindingResolutionException;
use Illuminate\Support\ServiceProvider;
use MirfalahTech\Laravel\Payment\Manager\PaymentManager;

class SenangPayServiceProvider extends ServiceProvider
{
    /**
     * @throws BindingResolutionException
     */
    public function boot()
    {
        /** @var Repository $config */
        $config = $this->app->make('config');

        $config->set('payment.gateway.senangpay', array_merge(
            [
                'endpoint' => env('SENANGPAY_ENDPOINT', 'https://app.senangpay.my/'),
                'merchant_id' => env('SENANGPAY_MERCHANT_ID'),
                'secret_key' => env('SENANGPAY_SECRET_KEY'),
                'key_map' => [
                    'status_id' => 'status_id',
                    'order_id' => 'order_id',
                    'hash' => 'hash'
                ],
            ],
            $config->get('payment.gateway.senangpay', [])
        ));

        $this->app->afterResolving('payment', function (PaymentManager $manager) use ($config) {
            $manager->extend('senangpay', function () use ($config) {
                return new SenangPayDriver(
                    $config->get('payment.gateway.senangpay.endpoint'),
                    $config->get('payment.gateway.senangpay.merchant_id'),
                    $config->get('payment.gateway.senangpay.secret_key')
                );
            });
        });
    }
}